@@ -1,8 +1,7 @@
 // "actual.h"
-// TAG: A
 // Actual objects to be instantiated.
 
-/*    Copyright (C) 1994  Steve Hardt
+/*    Copyright (C) 1994,1995,1996  Steve Hardt
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,10 +18,10 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
     Steve Hardt 
-    hardts@athena.mit.edu hardts@media.mit.edu
-    hardts@r4002.3dem.bioch.bcm.tmc.edu
-    2043 McClendon
-    Houston, TX 77030
+    hardts@mit.edu (valid until Nov. 1996)
+    hardts@netscape.com
+    hardts@alum.mit.edu
+    http://graphics.lcs.mit.edu/~hardts/xevil.html
 */
 
 #ifndef ACTUAL_H
@@ -42,20 +41,25 @@
 
 
 
+// Possible values for a ClassId.
 enum {A_None,A_Explosion,A_Fire,
       A_NProtection,A_TProtection,
       A_Trapdoor,A_Home,
-      A_Shell,A_SwapShell,A_Lance,A_FrogShell,A_Fireball,A_Missile,A_Star,
-      A_Grenade,
+      A_Shell,A_SwapShell,A_Lance,A_Laser,A_FrogShell,A_Fireball,A_Missile,
+      A_Star,
+      A_Grenade,A_Egg,
       A_Xit,A_Flag,
-      A_Rock,A_Weight,A_AltarOfSin,A_Doppel,
+      A_Rock,A_Weight,A_AltarOfSin,A_Doppel,A_Cloak,
       A_Transmogifier,A_MedKit,A_NShield,A_TShield,A_Bomb,
       A_Chainsaw,A_Pistol,A_MGun,A_Lancer,A_FThrower,A_Launcher,A_Grenades,
       A_Shotgun,A_Stars,
       A_Swapper,A_FrogGun,
-      A_Enforcer,A_Frog,A_Hero,A_Ninja,A_Alien,A_ChopperBoy,A_Lemming,
-      A_FireDemon,
-      A_SuicideButton}; // SuicideButton is not an actual class.
+      A_Enforcer,A_Frog,A_Hero,A_Ninja,A_Alien,A_RedHugger,A_GreenHugger,A_ChopperBoy,A_Lemming,
+      A_FireDemon,A_Walker,
+
+      // Identifiers for non-existent classes.
+      A_Fireballs, A_Lasers,
+      A_SuicideButton}; 
 
 
 
@@ -201,7 +205,7 @@ class AltarOfSin: public Heavy {
 
 
 
-class Doppel: public Item {
+class Doppel: public AutoUse {
   // Not a AutoUse because Lemmings and other funny things get doppelganged.
  public:
   Doppel(WorldP,LocatorP,const Pos &);
@@ -218,11 +222,31 @@ class Doppel: public Item {
  private:
   PhysicalP create_physical(const Area &,ClassId);
 
-  const static ItemContext context;
-  static ItemXdata xdata;
+  const static AutoUseContext context;
+  static AutoUseXdata xdata;
   static Stats stats;
 };
-  
+
+
+
+class Cloak: public AutoUse {
+public:
+  Cloak(WorldP,LocatorP,const Pos &);
+  static Size get_size_max();
+
+  virtual void use(PhysicalP);
+
+  static const Stats &get_stats() {return stats;}
+
+
+private:
+  static void attack_hook(PhysicalP p);
+
+  const static AutoUseContext context;
+  static AutoUseXdata xdata;
+  static Stats stats;
+};  
+
 
 
 class Transmogifier: public AutoUse {
@@ -371,6 +395,29 @@ class Lance: public Shot {
 
   static Size get_size(Dir);
 
+  static Dir compute_weapon_dir(ITcommand);
+  /* EFFECTS: Weapons firing a Lance shoot diagonal shots different than
+     most weapons. */
+
+
+ private:
+  static ShotXdata xdata;
+  const static ShotContext context;
+};
+
+
+
+class Laser: public Shot {
+ public:
+  Laser(WorldP w,LocatorP l,const Pos &pos,const Id &shooter,
+	Dir dir);
+
+  static Size get_size(Dir);
+
+  static Dir compute_weapon_dir(ITcommand);
+  /* EFFECTS: Weapons firing a Laser shoot diagonal shots different than
+     most weapons. */
+
 
  private:
   static ShotXdata xdata;
@@ -548,6 +595,30 @@ public:
 
 
 
+class Egg: public Falling {
+public:
+  Egg(WorldP w,LocatorP l,const Pos &pos,
+      const IntelOptions &ops,ITmask opMask);
+
+  static Size get_size_max();
+
+  virtual void act();
+
+
+ private:
+  Boolean open;
+  
+  // The intel options for the new Hugger.  Should be the same as for an
+  // Alien.
+  IntelOptions intelOptions;
+  ITmask intelOpMask;
+
+  const static FallingContext context;
+  static FallingXdata xdata;
+};
+
+
+
 // NOTE: Called Xit instead of Exit to avoid naming conflicts.
 class Xit: public Touchable {
 public:
@@ -620,9 +691,9 @@ protected:
 
   virtual Size get_shot_size(Dir);
 
-  virtual Dir compute_weapon_dir(ITcommand);
-  /* EFFECTS:  Returns the direction the weapon should be fired using c, or
-     CO_air if there is none. */
+  virtual Dir compute_weapon_dir(ITcommand c);
+  /* EFFECTS:  Returns the direction the weapon should be fired, 
+     {CO_R ... CO_UP_R}, or CO_air. */
 
   virtual void fire(const Id &id,ITcommand command);
   void _fire(const Id &id,Dir dir,Boolean setTimer = True,
@@ -918,6 +989,8 @@ public User
   virtual PhysicalP get_item_current();
   virtual void set_mapped_next(Boolean val);
 
+  virtual ClassId get_weapon_string(const char *&str);
+
   virtual void heal();
   
   virtual void act();
@@ -951,7 +1024,8 @@ private:
 
 
 
-class Ninja: public virtual Creature, public Sticky, public Fighter, public User {
+class Ninja: public virtual Creature, public Sticky, public Fighter, 
+public User {
  public:
   Ninja(WorldP w,LocatorP l,const Pos &pos);
   static Size get_size_max();
@@ -970,6 +1044,8 @@ class Ninja: public virtual Creature, public Sticky, public Fighter, public User
   virtual PhysicalP get_weapon_current();
   virtual PhysicalP get_item_current();
   virtual void set_mapped_next(Boolean val);
+
+  virtual ClassId get_weapon_string(const char *&str);
   
   virtual void act();
 
@@ -1027,6 +1103,52 @@ class Alien: public virtual Creature, public Sticky {
 
 
 
+class RedHugger: public virtual Creature, public Sticky, public Hugger {
+ public:
+  RedHugger(WorldP w,LocatorP l,const Pos &pos);
+
+  virtual Boolean is_sticky();
+  virtual Boolean is_hugger();
+
+  virtual void act();
+
+  virtual void collide(PhysicalP other);
+
+  
+ private:
+  const static CreatureContext creatureContext;
+  const static StickyContext stickyContext;
+  const static HuggerContext huggerContext;
+  static CreatureXdata creatureXdata;
+  static StickyXdata stickyXdata;
+  static HuggerXdata huggerXdata;
+};
+
+
+
+class GreenHugger: public virtual Creature, public Sticky, public Hugger {
+ public:
+  GreenHugger(WorldP w,LocatorP l,const Pos &pos);
+
+  virtual Boolean is_sticky();
+  virtual Boolean is_hugger();
+
+  virtual void act();
+
+  virtual void collide(PhysicalP other);
+
+  
+ private:
+  const static CreatureContext creatureContext;
+  const static StickyContext stickyContext;
+  const static HuggerContext huggerContext;
+  static CreatureXdata creatureXdata;
+  static StickyXdata stickyXdata;
+  static HuggerXdata huggerXdata;
+};
+
+
+
 class ChopperBoy: public virtual Creature, public Flying, public User {
  public:
   ChopperBoy(WorldP w,LocatorP l,const Pos &pos);
@@ -1043,6 +1165,8 @@ class ChopperBoy: public virtual Creature, public Flying, public User {
   virtual PhysicalP get_item_current();
   virtual void set_mapped_next(Boolean val);
   
+  virtual ClassId get_weapon_string(const char *&str);
+
   virtual void act();
 
   virtual void update();
@@ -1102,6 +1226,8 @@ class FireDemon: public virtual Creature, public Flying, public BuiltIn {
 
   virtual Boolean is_built_in();
 
+  virtual ClassId get_weapon_string(const char *&str);
+
   virtual Boolean ready();
 
   virtual void heat_attack(PhysicalP,int heat,Boolean secondary = False);
@@ -1134,6 +1260,66 @@ class FireDemon: public virtual Creature, public Flying, public BuiltIn {
   const static FlyingContext flyingContext;
   const static BuiltInContext builtInContext;
 };
+
+
+
+class Walker: public virtual Creature, public Walking, public User,
+public BuiltIn
+{
+public:
+  Walker(WorldP,LocatorP,const Pos &);
+  static Size get_size_max();
+
+  virtual Boolean is_walking();
+  virtual Boolean is_user();
+  virtual Boolean is_built_in();
+  
+  virtual int get_weapons_num();
+  virtual int get_items_num(); 
+  virtual PhysicalP get_weapon(int);
+  virtual PhysicalP get_item(int); 
+  virtual PhysicalP get_weapon_current();
+  virtual PhysicalP get_item_current();
+  virtual void set_mapped_next(Boolean val);
+
+  virtual ClassId get_weapon_string(const char *&str);
+
+  virtual void act();
+
+  virtual void update();
+  
+  virtual void die();
+
+  virtual void collide(PhysicalP other);
+
+  static const Stats &get_stats() {return stats;}
+  
+
+#ifndef PROTECTED_IS_PUBLIC
+ protected:
+#endif
+  virtual int get_anim_time();
+
+  virtual Dir compute_weapon_dir(ITcommand);
+
+  virtual Size get_shot_size(Dir);
+
+  virtual PhysicalP create_shot(const Pos &,Dir);
+
+  
+
+private:
+  static CreatureXdata creatureXdata;
+  static WalkingXdata walkingXdata;
+  static BuiltInXdata builtInXdata;
+  static UserXdata userXdata;
+  
+  const static CreatureContext creatureContext;
+  const static WalkingContext walkingContext;
+  const static BuiltInContext builtInContext;
+  const static UserContext userContext;
+  static Stats stats;
+};
 #endif
 
 
