@@ -1,8 +1,9 @@
-// "physical.h"  Interior nodes of the physical object tree.  These classes
+// "physical.h"  
+// Interior nodes of the physical object tree.  These classes
 // are never instantiated.
-// TAG: PH
 
-/*    Copyright (C) 1994  Steve Hardt
+
+/*    Copyright (C) 1994,1995,1996  Steve Hardt
 
       This program is free software; you can redistribute it and/or modify
       it under the terms of the GNU General Public License as published by
@@ -18,12 +19,13 @@
       along with this program; if not, write to the Free Software
       Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-      Steve Hardt 
-      hardts@athena.mit.edu hardts@media.mit.edu
-      hardts@r4002.3dem.bioch.bcm.tmc.edu
-      2043 McClendon
-      Houston, TX 77030
-      */
+    Steve Hardt 
+    hardts@mit.edu (valid until Nov. 1996)
+    hardts@netscape.com
+    hardts@alum.mit.edu
+    http://graphics.lcs.mit.edu/~hardts/xevil.html
+
+*/
 
 #ifndef PHYSICAL_H
 #define PHYSICAL_H
@@ -133,6 +135,8 @@ public:
   virtual Boolean is_sticky(); 
   virtual Boolean is_flying();
   virtual Boolean is_built_in();
+  virtual Boolean is_hugger();
+  virtual Boolean is_auto_use();
 
   Boolean get_mapped() {return mapped;}
 
@@ -268,6 +272,12 @@ public:
   virtual PhysicalP get_item_current();
   /* NOTE: Can return NULL. */
 
+  virtual ClassId get_weapon_string(const char *&str);
+  /* EFFECTS: str is set to point to a static string containing the string
+     for ClassId. */
+  /* NOTE: BuiltIn returns something for get_weapon_string(), but NULL
+     for get_weapon_current. */
+
   virtual Boolean ready();
 
 
@@ -281,7 +291,7 @@ protected:
 
   LocatorP get_locator() {return locator;}
 
-  const ITcommand &get_command() {return command;}
+  virtual ITcommand get_command();
   /* EFFECTS: Gets the command. */
   /* NOTE: command is not clocked. */
 
@@ -430,6 +440,11 @@ public:
   virtual const Area &get_area_next();
   /* NOTE: Should only be used by the Locator or in a protected context. */
 
+  Boolean is_invisible() {return invisible;}
+
+  virtual Boolean ignore_walls();
+  /* NOTE: Override this to make something able to "walk through walls." */
+
   virtual Vel get_vel();
   const Pos &get_raw_pos() {return rawPos;}
   virtual Dir get_dir();
@@ -456,9 +471,19 @@ public:
 
   void set_extra_vel_next(const Vel &vel) 
   {extraVelNext = vel; extraVelNextSet = True;}
+  void set_extra_vel_next(int zero) 
+    {assert (zero == 0); velNext.set_zero(); extraVelNextSet = True;}
 
   virtual void set_protection(const Id &);
 
+  void set_invisible_next(Boolean val) {invisibleNext = val;}
+
+  void set_attack_hook(void (*hook)(PhysicalP)) {attackHook = hook;}
+  /* NOTE: Overrides previous value if any. */
+
+  void attack_hook() {if (attackHook) attackHook(this);}
+  /* EFFECTS: Should be called whenever *this attacks anything. */
+
   virtual void corporeal_attack(PhysicalP killer,int damage); 
   virtual void heat_attack(PhysicalP,int heat,Boolean secondary = False);
 
@@ -481,7 +506,12 @@ protected:
   const MovingContext *get_moving_context() {return mc;}
   Vel get_vel_next() {return velNext;}
 
+  virtual int get_anim_time();
+  /* EFFECTS: Override this to make objects animate faster or slower.
+     Returns the number of turns per frame.  Default is MOVING_ANIM_TIME. */
+
   void set_vel(const Vel &v) {vel = v;}
+  void set_vel(int zero) {assert (zero == 0); vel.set_zero();}
 
   void set_dir(const Dir &d) {dir = d;}
   /* NOTE: Only used by Lance for initialization. */
@@ -528,6 +558,8 @@ private:
   Vel extraVel,extraVelNext; // Follows clock in non-standard way.
   Boolean hitWall,hitWallNext;
   Id protection;
+  Boolean invisible,invisibleNext; 
+  void (*attackHook)(PhysicalP);
 };
 typedef Moving *MovingP;
 
@@ -743,6 +775,7 @@ typedef Item *ItemP;
 // WARNING: All children of this class must be able to be used by a non-user.
 
 struct AutoUseContext {
+  Boolean humansOnly; // Only automatically used by humans.
   ItemContext itemContext;
 };
 typedef ItemXdata AutoUseXdata;
@@ -754,8 +787,14 @@ public:
 	  WorldP w,
 	  LocatorP l,
 	  const Pos &pos);
+
+  virtual Boolean is_auto_use();
   
   virtual void collide(PhysicalP);
+
+
+private:
+  const AutoUseContext *context;
 };
 
 
@@ -958,6 +997,9 @@ public:
   
   virtual void get_size_offset_next(Size &size,Size &offset,Dir dirNext);
 
+  virtual Boolean ignore_walls();
+  /* NOTE: We don't want cutters that are held to get bounced through walls. */
+
   virtual void set_ammo(int);
 
   virtual void follow_user(const Pos &,Dir);
@@ -1000,6 +1042,8 @@ struct CreatureContext {
   Speed acceleration;
   Health corpseHealth; // A positive number.
 
+  Boolean biological;
+
   Size deadSize;
   Size deadOffset;
   char *deadPixmapBits;
@@ -1034,6 +1078,9 @@ public:
   Stance get_stance() {return stance;}
   Boolean can_climb() {return canClimb;}
 
+  virtual ITcommand get_command();
+  /* NOTE: Will return ITnone if the Creature is stunned. */
+
   const Hanging &get_hanging() {return hanging;}
   /* EFFECTS: Get the corner that the object is hanging off of, or CO_AIR if
      not hanging off a corner. */
@@ -1043,6 +1090,29 @@ public:
 
   Grav get_grav() {return grav;}
 
+  Boolean get_alien_immune() {return !alienImmune.ready();}
+  /* EFFECTS: Is Creature immune to Alien and Hugger attacks. */
+
+  Boolean stunned() {return !stunTimer.ready();}
+
+  Boolean biological() {return cc->biological;}
+  /* EFFECTS: Is the Creature a biological organism.  E.g. as opposed to a 
+     machine. */
+
+  Boolean on_door();
+  /* NOTE: Convenience function to say whether *this is on top of a door. */
+
+  Boolean get_want_climb() {return wantClimb;}
+  void set_want_climb(Boolean val) {wantClimb = val;}
+  /* NOTE: Actually, ALL climbing should be in Creature. */
+
+  void stun_next(int time);
+  /* EFFECTS: Stun the creature for time turns. */
+
+  void set_alien_immune(int immTime) {alienImmune.set(immTime);}
+  /* EFFECTS: Will be immune to alien and hugger attacks for immTime turns. */
+  /* NOTE: Immediate. */
+
   virtual void set_quiet_death();
   /* NOTE: Calling set_quiet_death on a corpse destroys it. */
 
@@ -1075,6 +1145,8 @@ protected:
   virtual void get_size_offset_next(Size &size,Size &offset,Dir dirNext);
   /* MODIFIES: size, offset */
 
+  Boolean stunned_next() {return stunNext ? True : False;}
+
   void set_stance(const Stance &st) {stance = st;}
 
   void set_stance_next(const Stance &stance);
@@ -1114,9 +1186,15 @@ private:
   const CreatureContext *cc;
   Timer corpseTimer;
   CreatureXdata *creatureXdata;
-//  Id killerId;   Now in Intel.
+  Boolean wantClimb;  // actually, ALL climbing should be in Creature.
+  //  Id killerId;   Now in Intel.
   time_t birthTime;
   static Stats stats;
+
+  // If non-zero, indicates how many turns Creature should be stunned for.
+  int stunNext; 
+  Timer stunTimer;
+  Timer alienImmune;
 };
 typedef Creature *CreatureP;
 
@@ -1189,7 +1267,8 @@ class Hopping: public virtual Creature {
 // the User. 
 struct UserContext 
 {
-  int dummy;
+  Boolean usesWeapons;
+  Boolean usesItems;
 };  
 
 struct UserXdata {};
@@ -1211,6 +1290,10 @@ public:
   virtual PhysicalP get_item_current();
   /* NOTE: Can return NULL. */
 
+  virtual ClassId get_weapon_string(const char *&str);
+  /* EFFECTS: str is set to point to a static string containing the string
+     for ClassId. */
+
   virtual void set_mapped_next(Boolean val);
   /* EFFECTS: Bring current weapon in/out of scope and then change mapped. */
 
@@ -1248,6 +1331,8 @@ private:
   Id items[PH_ITEMS_MAX];
   int itemsNum;
   int itemCurrent; // itemCurrent == itemsNum if no selected item.
+
+  const UserContext *context;
 };
 typedef User *UserP;
 
@@ -1434,6 +1519,8 @@ private:
 struct BuiltInContext 
 {
   int shotTime;
+  ClassId weaponClassId;
+  const char *weaponStr;
 };  
 
 struct BuiltInXdata {};
@@ -1444,6 +1531,10 @@ public:
   
   virtual Boolean is_built_in();
 
+  virtual ClassId get_weapon_string(const char *&str);
+  /* EFFECTS: str is set to point to a static string containing the string
+     for ClassId. */
+
   virtual Boolean ready();
   /* EFFECTS: Is the BuiltIn ready to fire. */
 
@@ -1454,6 +1545,10 @@ public:
 #ifndef PROTECTED_IS_PUBLIC
 protected:
 #endif
+  virtual Dir compute_weapon_dir(ITcommand);
+  /* EFFECTS:  Returns the direction the weapon should be fired, 
+     {CO_R ... CO_UP_R}, or CO_air. */
+
   virtual Size get_shot_size(Dir) = 0;
 
   virtual PhysicalP create_shot(const Pos &,Dir) = 0;
@@ -1461,7 +1556,46 @@ protected:
 
 private:
   Timer shotTimer;
+  const BuiltInContext *context;
 };
 typedef BuiltIn *BuiltInP;
 
+
+
+////////// Hugger
+// Parent: Creature
+// Hugs onto any creature it collides with (except A_Alien and A_Hugger)
+// and stuns them, then turns them into an Alien.
+struct HuggerContext 
+{
+  // Which intel, the hugger's or the huggee's intel is implanted in the new
+  // Alien.
+  Boolean useHuggeeIntel; 
+};  
+
+struct HuggerXdata {};
+
+class Hugger: public virtual Creature {
+public:
+  Hugger(const HuggerContext &,HuggerXdata &);
+
+  static Size get_size_max();
+
+  virtual Boolean is_hugger();
+
+  virtual int get_drawing_level();
+
+  virtual void act();
+  void _act();
+
+  virtual void collide(PhysicalP other);
+
+
+private:
+  Id huggeeId;
+  const HuggerContext *context;
+};
+
 #endif
+
+
